package MyBlog::Controller::EntryCollection;

use strict;
use warnings;
use base qw(Catalyst::Controller::Atompub::Collection);

# ページあたりのエントリ数
my $ENTRIES_PER_PAGE = 10;
sub get_feed :Atompub(list) {
    my($self, $c) = @_;

    # フィード (XML::Atom::Feed) のひな型を取得する
    my $feed = $self->collection_resource->body;

    # コレクション URI
    my $uri = $self->collection_resource->uri;

    # リクエストされたページ (デフォルトは1ページ目)
    my $page = $c->req->param('page') || 1;

    # 検索時の開始レコード (offset) と取得数 (rows) を指定する
    my $attr = {
        offset   => ($page-1) * $ENTRIES_PER_PAGE,
        rows     => $ENTRIES_PER_PAGE,
        order_by => 'edited DESC',
    };

    # entries テーブルからエントリを取得する
    my $rs = $c->model('DBIC::Entries')->search({}, $attr);

    # フィードのひな形にエントリを追加する
    while (my $entry_resource = $rs->next) {
        my $entry = XML::Atom::Entry->new(\$entry_resource->xml);
        $feed->add_entry($entry);
    }

    # 最初のページへのリンクを追加する．
    $feed->first_link($uri);

    # 前後のページへのリンクを追加する
    $feed->previous_link("$uri?page=".($page-1)) if $page > 1;
    $feed->next_link("$uri?page=".($page+1)) if $rs->count >= $ENTRIES_PER_PAGE;

    # 成功したら true を返す
    return 1;
}

use Digest::MD5 qw(md5_hex);
sub create_resource :Atompub(create) {
    my($self, $c) = @_;

    # スーパークラスが自動生成した URI と更新日時を取得する
    my $uri    = $self->entry_resource->uri;
    my $edited = $self->edited->epoch; # (UNIX time 形式に変換)

    # POST されたエントリ本体を取得する (XML::Atom::Entry)
    my $entry = $self->entry_resource->body;

    # entries テーブルにエントリとメタデータを格納する
    $c->model('DBIC::Entries')->update_or_create({
        edited => $edited,
        uri    => $uri,
        xml    => $entry->as_xml,
        etag   => md5_hex($entry->as_xml),
    });

    # 成功したら true を返す
    return 1;
}

sub get_resource :Atompub(read) {
    my( $self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # entries テーブルからエントリを検索する
    my $rs = $c->model('DBIC::Entries')->find({ uri => $uri })
        || return $self->error($c, 404, 'Entry does not exist');

    # エントリリソースをセットする
    my $entry = XML::Atom::Entry->new(\$rs->xml);
    $self->entry_resource->body($entry);

    # 成功したら true を返す
    return 1;
}

sub update_entry :Atompub(update) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # スーパークラスが生成した更新日時を取得する
    my $edited = $self->edited->epoch; # (UNIX time 形式に変換)

    # PUT されたエントリ本体を取得する (XML::Atom::Entry)
    my $entry = $self->entry_resource->body;

    # entries テーブルからエントリを検索し，更新する
    $c->model('DBIC::Entries')->search({ uri => $uri })->update({
        edited => $edited,
        xml    => $entry->as_xml,
        etag   => md5_hex($entry->as_xml),
    });

    # 成功したら true を返す
    return 1;
}

sub delete_entry :Atompub(delete) {
    my($self, $c) = @_;

    # リクエストされた URI
    my $uri = $c->req->uri;

    # entries テーブルからエントリを削除する
    $c->model('DBIC::Entries')->search({ uri => $uri })->delete;

    # 成功したら true を返す
    return 1;
}

use Time::HiRes qw(gettimeofday);
sub make_edit_uri {
    my($self, $c, @args) = @_;

    # スーパークラスの make_edit_uri を呼び出す
    my $uri = $self->SUPER::make_edit_uri($c, @args);

    # URI がすでに存在するときは，拡張子の前に UNIX time を挿入する
    if ($c->model('DBIC::Entries')->find({ uri => $uri })) {
        my $t = join '', gettimeofday;
        $uri =~ s{(\.[^.]+)$}{-$t$1};
    }

    # 空のリソースを作成し，URI を予約する
    $c->model('DBIC::Entries')->create({ uri => $uri });

    return $uri;
}

sub find_version {
    # $uri はリクエストされた URI
    my($self, $c, $uri) = @_;

    # $uri に対応するリソースを取得する (リソースが存在しなければ空配列を返す)
    my $rs = $c->model('DBIC::Entries')->find({ uri => $uri }) || return;

    # ETag をハッシュとして返す
    return (etag => $rs->etag);

    # ETag と Last-Modified をハッシュとして返すこともできる
    #return (etag => $rs->etag, last_modified => $rs->last_modified);
}

=head1 NAME

MyBlog::Controller::EntryCollection - Atom Collection Resources

=head1 SYNOPSIS

See L<MyBlog>.

=head1 DESCRIPTION

Atom Collection Resources class.

=head1 AUTHOR

Takeru INOUE

=head1 LICENSE

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
